package org.nakedobjects.nos.remote;

import java.util.HashMap;
import java.util.Map;

import org.nakedobjects.noa.security.Session;
import org.nakedobjects.nof.reflect.peer.MemberIdentifier;
import org.nakedobjects.nof.reflect.remote.data.Distribution;
import org.nakedobjects.nof.reflect.security.AuthorisationManager;


public class ProxyAuthorisationManager implements AuthorisationManager {

    private final Distribution connection;
    private final Map editableCache = new HashMap();
    private final Map visibleCache = new HashMap();

    public ProxyAuthorisationManager(final Distribution connection) {
        this.connection = connection;
    }

    private boolean remoteAuthUsability(final Session session, final String idString) {
        return connection.authoriseUsability(session, idString);
    }

    private boolean remoteAuthVisibility(final Session session, final String idString) {
        return connection.authoriseVisibility(session, idString);
    }

    private boolean isAuthorised(final Session session, final MemberIdentifier identifier, final Map cache) {
        final String idString = identifier.toIdentityString(MemberIdentifier.CLASS_NAME_PARMS);
        if (!cache.containsKey(idString)) {
            final boolean allow = cache == editableCache ? remoteAuthUsability(session, idString) : remoteAuthVisibility(session,
                    idString);
            cache.put(idString, new Boolean(allow));
            return allow;
        }
        return ((Boolean) cache.get(idString)).booleanValue();
    }

    public boolean isEditable(final Session session, final MemberIdentifier identifier) {
        return isAuthorised(session, identifier, editableCache);
    }

    public boolean isVisible(final Session session, final MemberIdentifier identifier) {
        return isAuthorised(session, identifier, visibleCache);
    }

    public void init() {}

    public void shutdown() {}
}
