package org.nakedobjects.nos.remote.command;

import org.nakedobjects.noa.adapter.Oid;
import org.nakedobjects.noa.security.Session;
import org.nakedobjects.noa.util.ByteDecoder;
import org.nakedobjects.noa.util.ByteEncoder;
import org.nakedobjects.nof.core.util.ToString;
import org.nakedobjects.nof.reflect.remote.data.Distribution;
import org.nakedobjects.nof.reflect.remote.data.ObjectData;

public class GetObject extends AbstractRequest {
    private static final long serialVersionUID = 1L;
    private final Oid oid;
    private final String specificationName;

    public GetObject(Session session, Oid oid, String specificationName) {
        super(session);
        this.oid = oid;
        this.specificationName = specificationName;
    }

    public GetObject(final ByteDecoder decoder) {
        super(decoder);
        oid = (Oid) decoder.getObject();
        specificationName = decoder.getString();
    }

    protected void doEncode(final ByteEncoder encoder) {
        encoder.add(oid);
        encoder.add(specificationName);
    }
    
    public ObjectData getObject() {
        return (ObjectData) response;
    }

    public void execute(final Distribution distribution) {
        ObjectData object = distribution.getObject(session, oid, specificationName);
        setResponse(object);
    }

    public String toString() {
        ToString str = new ToString(this);
        str.append("oid", oid);
        str.append("spec", specificationName);
        return str.toString();
    }
}


// Copyright (c) Naked Objects Group Ltd.
